module Data.Time.Compat (
    module Data.Time.Calendar.Compat,
    module Data.Time.Clock.Compat,
    module Data.Time.LocalTime.Compat,
    module Data.Time.Format.Compat,
    ) where

import Data.Time.Orphans ()

import Data.Time.Calendar.Compat
import Data.Time.Clock.Compat
import Data.Time.LocalTime.Compat
import Data.Time.Format.Compat
